package com.supervision.knowsub.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.knowsub.model.SystemBase;
import com.supervision.knowsub.model.SystemBaseDeptRelation;
import com.supervision.knowsub.service.ApplicationSubLibraryService;
import com.supervision.knowsub.service.SystemBaseDeptRelationService;
import com.supervision.knowsub.service.SystemBaseService;
import com.supervision.knowsub.vo.sublibrary.SubLibraryReqVo;
import com.supervision.knowsub.vo.sublibrary.SubLibraryResVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class ApplicationSubLibraryServiceImpl implements ApplicationSubLibraryService {

    private final SystemBaseService systemBaseService;


    private final SystemBaseDeptRelationService systemBaseDeptRelationService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveSubLibrary(SubLibraryReqVo subLibraryReqVo) {

        Assert.notEmpty(subLibraryReqVo.getBaseCode(), "子库编码不能为空");
        Assert.notEmpty(subLibraryReqVo.getBaseName(), "子库名称不能为空");
        Assert.notEmpty(subLibraryReqVo.getDeptIds(), "关联部门不能为空");


        Long count = systemBaseService.lambdaQuery().eq(SystemBase::getBaseCode, subLibraryReqVo.getBaseCode())
                .or(query -> query.eq(SystemBase::getBaseName, subLibraryReqVo.getBaseName())).count();
        Assert.isTrue(count == 0, "子库编码或子库名称已存在");

        // 保存应用子库信息
        SystemBase systemBase = new SystemBase();
        systemBase.setBaseCode(subLibraryReqVo.getBaseCode());
        systemBase.setBaseName(subLibraryReqVo.getBaseName());
        systemBaseService.save(systemBase);

        // 保存应用子库管理部门信息
        systemBaseDeptRelationService.saveBatch(subLibraryReqVo.getDeptIds().stream().map(deptId -> {
            SystemBaseDeptRelation systemBaseDeptRelation = new SystemBaseDeptRelation();
            systemBaseDeptRelation.setBaseId(systemBase.getId());
            systemBaseDeptRelation.setDeptId(deptId);
            return systemBaseDeptRelation;
        }).toList());

        return systemBase.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateSubLibrary(SubLibraryReqVo subLibraryReqVo) {

        Assert.notEmpty(subLibraryReqVo.getBaseCode(), "子库编码不能为空");
        Assert.notEmpty(subLibraryReqVo.getBaseName(), "子库名称不能为空");
        Assert.notEmpty(subLibraryReqVo.getDeptIds(), "关联部门不能为空");

        SystemBase dbSystemBase = systemBaseService.getById(subLibraryReqVo.getId());
        Assert.isTrue(Objects.nonNull(dbSystemBase), "子库不存在");
        if (!StrUtil.equals(dbSystemBase.getBaseCode(), subLibraryReqVo.getBaseCode())){
            // 子库编码修改
            Long count = systemBaseService.lambdaQuery().eq(SystemBase::getBaseCode, subLibraryReqVo.getBaseCode()).count();
            Assert.isTrue(count == 0, "子库编码已存在");
        }


        // 更新应用子库信息
        SystemBase systemBase = new SystemBase();
        systemBase.setId(subLibraryReqVo.getId());
        systemBase.setBaseName(subLibraryReqVo.getBaseName());
        systemBase.setBaseCode(subLibraryReqVo.getBaseCode());
        systemBaseService.updateById(systemBase);

        // 更新应用子库管理部门信息(先删除再新增)
        systemBaseDeptRelationService.lambdaUpdate().eq(SystemBaseDeptRelation::getBaseId, systemBase.getId()).remove();
        systemBaseDeptRelationService.saveBatch(subLibraryReqVo.getDeptIds().stream().map(deptId -> {
            SystemBaseDeptRelation systemBaseDeptRelation = new SystemBaseDeptRelation();
            systemBaseDeptRelation.setBaseId(systemBase.getId());
            systemBaseDeptRelation.setDeptId(deptId);
            return systemBaseDeptRelation;
        }).toList());

    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteSubLibrary(String id) {
        Assert.notEmpty(id, "子库id不能为空");
        systemBaseService.removeById(id);
        systemBaseDeptRelationService.lambdaUpdate().eq(SystemBaseDeptRelation::getBaseId, id).remove();
        return true;
    }

    @Override
    public IPage<SubLibraryResVo> listSubLibrary(SubLibraryReqVo subLibraryReqVo,Integer pageNum,Integer pageSize) {

        return systemBaseService.listSubLibrary(subLibraryReqVo, pageNum, pageSize);
    }

    @Override
    public List<SystemBase> listAllSubLibrary() {

        return systemBaseService.list();
    }
}
